import { Address, ContractState, TestContractResult, HexString, ContractFactory, TestContractParams, ContractInstance } from "@alephium/web3";
export declare namespace AttestTokenHandlerTypes {
    type Fields = {
        governance: HexString;
        localTokenBridge: HexString;
        targetChainId: bigint;
        targetTokenBridge: HexString;
        receivedSequence: bigint;
        isLocalHandler: boolean;
    };
    type State = ContractState<Fields>;
}
declare class Factory extends ContractFactory<AttestTokenHandlerInstance, AttestTokenHandlerTypes.Fields> {
    consts: {
        Path: {
            AttestTokenHandler: string;
            TokenBridgeForChain: string;
            TokenPool: string;
        };
        ErrorCodes: {
            InvalidEmitChainId: bigint;
            InvalidEmitAddress: bigint;
            InvalidMessageSize: bigint;
            InvalidSequence: bigint;
            InvalidModule: bigint;
            InvalidActionId: bigint;
            InvalidVersion: bigint;
            InvalidGuardianSetIndex: bigint;
            InvalidGuardianSetSize: bigint;
            InvalidSignatureSize: bigint;
            InvalidSignatureGuardianIndex: bigint;
            InvalidSignature: bigint;
            GuardianSetExpired: bigint;
            InvalidTargetChainId: bigint;
            ContractStateMismatch: bigint;
            InvalidRegisterChainMessage: bigint;
            InvalidTokenId: bigint;
            InvalidNonceSize: bigint;
            TokenNotExist: bigint;
            InvalidTransferTargetChain: bigint;
            InvalidDestroyUnexecutedSequenceMessage: bigint;
            InvalidCaller: bigint;
            ArbiterFeeLessThanAmount: bigint;
            InvalidAttestTokenMessage: bigint;
            InvalidPayloadId: bigint;
            InvalidTransferMessage: bigint;
            ExpectRemoteToken: bigint;
            InvalidConsistencyLevel: bigint;
            InvalidUpdateRefundAddressMessage: bigint;
            TransferAmountLessThanMessageFee: bigint;
            InvalidAttestTokenArg: bigint;
            InvalidAttestTokenHandler: bigint;
            NotSupported: bigint;
        };
        PayloadId: {
            Transfer: string;
            AttestToken: string;
        };
    };
    at(address: string): AttestTokenHandlerInstance;
    tests: {
        parseAttestToken: (params: TestContractParams<AttestTokenHandlerTypes.Fields, {
            vaa: HexString;
        }>) => Promise<TestContractResult<[HexString, HexString, HexString, bigint, bigint]>>;
        createLocalTokenPool: (params: TestContractParams<AttestTokenHandlerTypes.Fields, {
            vaa: HexString;
            payer: Address;
            createContractAlphAmount: bigint;
            tokenAmount: bigint;
        }>) => Promise<TestContractResult<null>>;
        createRemoteTokenPool: (params: TestContractParams<AttestTokenHandlerTypes.Fields, {
            vaa: HexString;
            payer: Address;
            createContractAlphAmount: bigint;
        }>) => Promise<TestContractResult<null>>;
        updateRemoteTokenPool: (params: TestContractParams<AttestTokenHandlerTypes.Fields, {
            vaa: HexString;
        }>) => Promise<TestContractResult<null>>;
        removeTrailingZeros: (params: TestContractParams<AttestTokenHandlerTypes.Fields, {
            bytes: HexString;
        }>) => Promise<TestContractResult<HexString>>;
    };
}
export declare const AttestTokenHandler: Factory;
export declare class AttestTokenHandlerInstance extends ContractInstance {
    constructor(address: Address);
    fetchState(): Promise<AttestTokenHandlerTypes.State>;
}
export {};
