import { Address, ContractState, TestContractResult, HexString, ContractFactory, CallContractParams, CallContractResult, TestContractParams, ContractInstance } from "@alephium/web3";
export declare namespace SequenceTestTypes {
    type Fields = {
        start: bigint;
        firstNext256: bigint;
        secondNext256: bigint;
        unexecutedSequenceTemplateId: HexString;
    };
    type State = ContractState<Fields>;
    interface CallMethodTable {
        check: {
            params: CallContractParams<{
                seq: bigint;
            }>;
            result: CallContractResult<boolean>;
        };
    }
    type CallMethodParams<T extends keyof CallMethodTable> = CallMethodTable[T]["params"];
    type CallMethodResult<T extends keyof CallMethodTable> = CallMethodTable[T]["result"];
    type MultiCallParams = Partial<{
        [Name in keyof CallMethodTable]: CallMethodTable[Name]["params"];
    }>;
    type MultiCallResults<T extends MultiCallParams> = {
        [MaybeName in keyof T]: MaybeName extends keyof CallMethodTable ? CallMethodTable[MaybeName]["result"] : undefined;
    };
}
declare class Factory extends ContractFactory<SequenceTestInstance, SequenceTestTypes.Fields> {
    consts: {
        ErrorCodes: {
            InvalidEmitChainId: bigint;
            InvalidEmitAddress: bigint;
            InvalidMessageSize: bigint;
            InvalidSequence: bigint;
            InvalidModule: bigint;
            InvalidActionId: bigint;
            InvalidVersion: bigint;
            InvalidGuardianSetIndex: bigint;
            InvalidGuardianSetSize: bigint;
            InvalidSignatureSize: bigint;
            InvalidSignatureGuardianIndex: bigint;
            InvalidSignature: bigint;
            GuardianSetExpired: bigint;
            InvalidTargetChainId: bigint;
            ContractStateMismatch: bigint;
            InvalidRegisterChainMessage: bigint;
            InvalidTokenId: bigint;
            InvalidNonceSize: bigint;
            TokenNotExist: bigint;
            InvalidTransferTargetChain: bigint;
            InvalidDestroyUnexecutedSequenceMessage: bigint;
            InvalidCaller: bigint;
            ArbiterFeeLessThanAmount: bigint;
            InvalidAttestTokenMessage: bigint;
            InvalidPayloadId: bigint;
            InvalidTransferMessage: bigint;
            ExpectRemoteToken: bigint;
            InvalidConsistencyLevel: bigint;
            InvalidUpdateRefundAddressMessage: bigint;
            TransferAmountLessThanMessageFee: bigint;
            InvalidAttestTokenArg: bigint;
            InvalidAttestTokenHandler: bigint;
            NotSupported: bigint;
        };
    };
    at(address: string): SequenceTestInstance;
    tests: {
        setExecuted: (params: TestContractParams<SequenceTestTypes.Fields, {
            offset: bigint;
            current: bigint;
        }>) => Promise<TestContractResult<bigint>>;
        compact: (params: Omit<TestContractParams<SequenceTestTypes.Fields, never>, "testArgs">) => Promise<TestContractResult<null>>;
        checkSequenceInSubContract: (params: TestContractParams<SequenceTestTypes.Fields, {
            seq: bigint;
        }>) => Promise<TestContractResult<null>>;
        checkSequence: (params: TestContractParams<SequenceTestTypes.Fields, {
            seq: bigint;
        }>) => Promise<TestContractResult<boolean>>;
        check: (params: TestContractParams<SequenceTestTypes.Fields, {
            seq: bigint;
        }>) => Promise<TestContractResult<boolean>>;
    };
}
export declare const SequenceTest: Factory;
export declare class SequenceTestInstance extends ContractInstance {
    constructor(address: Address);
    fetchState(): Promise<SequenceTestTypes.State>;
    methods: {
        check: (params: SequenceTestTypes.CallMethodParams<"check">) => Promise<SequenceTestTypes.CallMethodResult<"check">>;
    };
    multicall<Calls extends SequenceTestTypes.MultiCallParams>(calls: Calls): Promise<SequenceTestTypes.MultiCallResults<Calls>>;
}
export {};
