import { Address, ContractState, TestContractResult, HexString, ContractFactory, CallContractParams, CallContractResult, TestContractParams, ContractInstance } from "@alephium/web3";
export declare namespace TokenBridgeFactoryTypes {
    type Fields = {
        localTokenPoolTemplateId: HexString;
        remoteTokenPoolTemplateId: HexString;
        tokenBridgeForChainTemplateId: HexString;
        attestTokenHandlerTemplateId: HexString;
        unexecutedSequenceTemplateId: HexString;
    };
    type State = ContractState<Fields>;
    interface CallMethodTable {
        getLocalTokenPoolTemplateId: {
            params: Omit<CallContractParams<{}>, "args">;
            result: CallContractResult<HexString>;
        };
        getRemoteTokenPoolTemplateId: {
            params: Omit<CallContractParams<{}>, "args">;
            result: CallContractResult<HexString>;
        };
        getTokenBridgeForChainTemplateId: {
            params: Omit<CallContractParams<{}>, "args">;
            result: CallContractResult<HexString>;
        };
        getAttestTokenHandlerTemplateId: {
            params: Omit<CallContractParams<{}>, "args">;
            result: CallContractResult<HexString>;
        };
        getUnexecutedSequenceTemplateId: {
            params: Omit<CallContractParams<{}>, "args">;
            result: CallContractResult<HexString>;
        };
        parseContractUpgrade: {
            params: CallContractParams<{
                payload: HexString;
            }>;
            result: CallContractResult<[HexString, HexString, HexString, HexString]>;
        };
    }
    type CallMethodParams<T extends keyof CallMethodTable> = CallMethodTable[T]["params"];
    type CallMethodResult<T extends keyof CallMethodTable> = CallMethodTable[T]["result"];
    type MultiCallParams = Partial<{
        [Name in keyof CallMethodTable]: CallMethodTable[Name]["params"];
    }>;
    type MultiCallResults<T extends MultiCallParams> = {
        [MaybeName in keyof T]: MaybeName extends keyof CallMethodTable ? CallMethodTable[MaybeName]["result"] : undefined;
    };
}
declare class Factory extends ContractFactory<TokenBridgeFactoryInstance, TokenBridgeFactoryTypes.Fields> {
    consts: {
        ErrorCodes: {
            InvalidEmitChainId: bigint;
            InvalidEmitAddress: bigint;
            InvalidMessageSize: bigint;
            InvalidSequence: bigint;
            InvalidModule: bigint;
            InvalidActionId: bigint;
            InvalidVersion: bigint;
            InvalidGuardianSetIndex: bigint;
            InvalidGuardianSetSize: bigint;
            InvalidSignatureSize: bigint;
            InvalidSignatureGuardianIndex: bigint;
            InvalidSignature: bigint;
            GuardianSetExpired: bigint;
            InvalidTargetChainId: bigint;
            ContractStateMismatch: bigint;
            InvalidRegisterChainMessage: bigint;
            InvalidTokenId: bigint;
            InvalidNonceSize: bigint;
            TokenNotExist: bigint;
            InvalidTransferTargetChain: bigint;
            InvalidDestroyUnexecutedSequenceMessage: bigint;
            InvalidCaller: bigint;
            ArbiterFeeLessThanAmount: bigint;
            InvalidAttestTokenMessage: bigint;
            InvalidPayloadId: bigint;
            InvalidTransferMessage: bigint;
            ExpectRemoteToken: bigint;
            InvalidConsistencyLevel: bigint;
            InvalidUpdateRefundAddressMessage: bigint;
            TransferAmountLessThanMessageFee: bigint;
            InvalidAttestTokenArg: bigint;
            InvalidAttestTokenHandler: bigint;
            NotSupported: bigint;
        };
    };
    at(address: string): TokenBridgeFactoryInstance;
    tests: {
        getLocalTokenPoolTemplateId: (params: Omit<TestContractParams<TokenBridgeFactoryTypes.Fields, never>, "testArgs">) => Promise<TestContractResult<HexString>>;
        getRemoteTokenPoolTemplateId: (params: Omit<TestContractParams<TokenBridgeFactoryTypes.Fields, never>, "testArgs">) => Promise<TestContractResult<HexString>>;
        getTokenBridgeForChainTemplateId: (params: Omit<TestContractParams<TokenBridgeFactoryTypes.Fields, never>, "testArgs">) => Promise<TestContractResult<HexString>>;
        getAttestTokenHandlerTemplateId: (params: Omit<TestContractParams<TokenBridgeFactoryTypes.Fields, never>, "testArgs">) => Promise<TestContractResult<HexString>>;
        getUnexecutedSequenceTemplateId: (params: Omit<TestContractParams<TokenBridgeFactoryTypes.Fields, never>, "testArgs">) => Promise<TestContractResult<HexString>>;
        parseContractUpgrade: (params: TestContractParams<TokenBridgeFactoryTypes.Fields, {
            payload: HexString;
        }>) => Promise<TestContractResult<[HexString, HexString, HexString, HexString]>>;
    };
}
export declare const TokenBridgeFactory: Factory;
export declare class TokenBridgeFactoryInstance extends ContractInstance {
    constructor(address: Address);
    fetchState(): Promise<TokenBridgeFactoryTypes.State>;
    methods: {
        getLocalTokenPoolTemplateId: (params?: Omit<CallContractParams<{}>, "args"> | undefined) => Promise<TokenBridgeFactoryTypes.CallMethodResult<"getLocalTokenPoolTemplateId">>;
        getRemoteTokenPoolTemplateId: (params?: Omit<CallContractParams<{}>, "args"> | undefined) => Promise<TokenBridgeFactoryTypes.CallMethodResult<"getRemoteTokenPoolTemplateId">>;
        getTokenBridgeForChainTemplateId: (params?: Omit<CallContractParams<{}>, "args"> | undefined) => Promise<TokenBridgeFactoryTypes.CallMethodResult<"getTokenBridgeForChainTemplateId">>;
        getAttestTokenHandlerTemplateId: (params?: Omit<CallContractParams<{}>, "args"> | undefined) => Promise<TokenBridgeFactoryTypes.CallMethodResult<"getAttestTokenHandlerTemplateId">>;
        getUnexecutedSequenceTemplateId: (params?: Omit<CallContractParams<{}>, "args"> | undefined) => Promise<TokenBridgeFactoryTypes.CallMethodResult<"getUnexecutedSequenceTemplateId">>;
        parseContractUpgrade: (params: TokenBridgeFactoryTypes.CallMethodParams<"parseContractUpgrade">) => Promise<TokenBridgeFactoryTypes.CallMethodResult<"parseContractUpgrade">>;
    };
    multicall<Calls extends TokenBridgeFactoryTypes.MultiCallParams>(calls: Calls): Promise<TokenBridgeFactoryTypes.MultiCallResults<Calls>>;
}
export {};
