"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getOriginalAssetTerra = exports.getOriginalAssetSol = exports.getOriginalAssetEth = void 0;
const web3_js_1 = require("@solana/web3.js");
const ethers_1 = require("ethers");
const utils_1 = require("ethers/lib/utils");
const __1 = require("..");
const ethers_contracts_1 = require("../ethers-contracts");
const wasm_1 = require("../solana/wasm");
const utils_2 = require("../utils");
const getIsWrappedAsset_1 = require("./getIsWrappedAsset");
/**
 * Returns a origin chain and asset address on {originChain} for a provided Wormhole wrapped address
 * @param tokenBridgeAddress
 * @param provider
 * @param wrappedAddress
 * @returns
 */
function getOriginalAssetEth(tokenBridgeAddress, provider, wrappedAddress, tokenId, lookupChain) {
    return __awaiter(this, void 0, void 0, function* () {
        const isWrapped = yield getIsWrappedAsset_1.getIsWrappedAssetEth(tokenBridgeAddress, provider, wrappedAddress);
        if (isWrapped) {
            const token = ethers_contracts_1.TokenImplementation__factory.connect(wrappedAddress, provider);
            const chainId = (yield token.chainId()); // origin chain
            const assetAddress = yield token.nativeContract(); // origin address
            return {
                isWrapped: true,
                chainId,
                assetAddress: chainId === utils_2.CHAIN_ID_SOLANA
                    ? utils_1.arrayify(ethers_1.BigNumber.from(tokenId))
                    : utils_1.arrayify(assetAddress),
                tokenId, // tokenIds are maintained across EVM chains
            };
        }
        return {
            isWrapped: false,
            chainId: utils_2.coalesceChainId(lookupChain),
            assetAddress: utils_1.zeroPad(utils_1.arrayify(wrappedAddress), 32),
            tokenId,
        };
    });
}
exports.getOriginalAssetEth = getOriginalAssetEth;
/**
 * Returns a origin chain and asset address on {originChain} for a provided Wormhole wrapped address
 * @param connection
 * @param tokenBridgeAddress
 * @param mintAddress
 * @returns
 */
function getOriginalAssetSol(connection, tokenBridgeAddress, mintAddress) {
    return __awaiter(this, void 0, void 0, function* () {
        if (mintAddress) {
            // TODO: share some of this with getIsWrappedAssetSol, like a getWrappedMetaAccountAddress or something
            const { parse_wrapped_meta, wrapped_meta_address } = yield wasm_1.importNftWasm();
            const wrappedMetaAddress = wrapped_meta_address(tokenBridgeAddress, new web3_js_1.PublicKey(mintAddress).toBytes());
            const wrappedMetaAddressPK = new web3_js_1.PublicKey(wrappedMetaAddress);
            const wrappedMetaAccountInfo = yield connection.getAccountInfo(wrappedMetaAddressPK);
            if (wrappedMetaAccountInfo) {
                const parsed = parse_wrapped_meta(wrappedMetaAccountInfo.data);
                const token_id_arr = parsed.token_id;
                const token_id_bytes = [];
                for (let elem of token_id_arr.reverse()) {
                    token_id_bytes.push(...bigToUint8Array(elem));
                }
                const token_id = ethers_1.BigNumber.from(token_id_bytes).toString();
                return {
                    isWrapped: true,
                    chainId: parsed.chain,
                    assetAddress: parsed.token_address,
                    tokenId: token_id,
                };
            }
        }
        try {
            return {
                isWrapped: false,
                chainId: utils_2.CHAIN_ID_SOLANA,
                assetAddress: new web3_js_1.PublicKey(mintAddress).toBytes(),
            };
        }
        catch (e) { }
        return {
            isWrapped: false,
            chainId: utils_2.CHAIN_ID_SOLANA,
            assetAddress: new Uint8Array(32),
        };
    });
}
exports.getOriginalAssetSol = getOriginalAssetSol;
// Derived from https://www.jackieli.dev/posts/bigint-to-uint8array/
const big0 = BigInt(0);
const big1 = BigInt(1);
const big8 = BigInt(8);
function bigToUint8Array(big) {
    if (big < big0) {
        const bits = (BigInt(big.toString(2).length) / big8 + big1) * big8;
        const prefix1 = big1 << bits;
        big += prefix1;
    }
    let hex = big.toString(16);
    if (hex.length % 2) {
        hex = "0" + hex;
    }
    else if (hex[0] === "8") {
        // maximum positive need to prepend 0 otherwise resuts in negative number
        hex = "00" + hex;
    }
    const len = hex.length / 2;
    const u8 = new Uint8Array(len);
    var i = 0;
    var j = 0;
    while (i < len) {
        u8[i] = parseInt(hex.slice(j, j + 2), 16);
        i += 1;
        j += 2;
    }
    return u8;
}
function getOriginalAssetTerra(client, wrappedAddress) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const result = yield client.wasm.contractQuery(wrappedAddress, {
                wrapped_asset_info: {},
            });
            if (result) {
                return {
                    isWrapped: true,
                    chainId: result.asset_chain,
                    assetAddress: new Uint8Array(Buffer.from(result.asset_address, "base64")),
                };
            }
        }
        catch (e) { }
        return {
            isWrapped: false,
            chainId: utils_2.CHAIN_ID_TERRA,
            assetAddress: utils_1.zeroPad(__1.canonicalAddress(wrappedAddress), 32),
        };
    });
}
exports.getOriginalAssetTerra = getOriginalAssetTerra;
