"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transferFromAlgorand = exports.transferFromSolana = exports.transferNativeSol = exports.transferFromTerra = exports.transferFromEthNative = exports.transferFromEth = exports.approveEth = exports.getAllowanceEth = exports.transferRemoteTokenFromAlph = exports.transferLocalTokenFromAlph = void 0;
const web3_1 = require("@alephium/web3");
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const terra_js_1 = require("@terra-money/terra.js");
const algosdk_1 = require("algosdk");
const __1 = require("..");
const scripts_1 = require("../alephium-contracts/ts/scripts");
const algorand_1 = require("../algorand");
const bridge_1 = require("../bridge");
const ethers_contracts_1 = require("../ethers-contracts");
const solana_1 = require("../solana");
const wasm_1 = require("../solana/wasm");
const utils_1 = require("../utils");
const bigint_1 = require("../utils/bigint");
function transferLocalTokenFromAlph(signerProvider, tokenBridgeId, fromAddress, localTokenId, toChainId, toAddress, tokenAmount, messageFee, arbiterFee, consistencyLevel, nonce) {
    return __awaiter(this, void 0, void 0, function* () {
        const nonceHex = (typeof nonce !== "undefined") ? nonce : utils_1.createNonce().toString('hex');
        return scripts_1.TransferLocal.execute(signerProvider, {
            initialFields: {
                tokenBridge: tokenBridgeId,
                fromAddress: fromAddress,
                localTokenId: localTokenId,
                alphChainId: BigInt(utils_1.CHAIN_ID_ALEPHIUM),
                toChainId: BigInt(toChainId),
                toAddress: toAddress,
                tokenAmount: tokenAmount,
                arbiterFee: arbiterFee,
                nonce: nonceHex,
                consistencyLevel: BigInt(consistencyLevel)
            },
            attoAlphAmount: localTokenId === web3_1.ALPH_TOKEN_ID ? messageFee + tokenAmount : messageFee,
            tokens: localTokenId === web3_1.ALPH_TOKEN_ID
                ? []
                : [{ id: localTokenId, amount: tokenAmount }, { id: web3_1.ALPH_TOKEN_ID, amount: web3_1.DUST_AMOUNT * BigInt(2) }]
        });
    });
}
exports.transferLocalTokenFromAlph = transferLocalTokenFromAlph;
function transferRemoteTokenFromAlph(signerProvider, tokenBridgeId, fromAddress, tokenPoolId, remoteTokenId, tokenChainId, toChainId, toAddress, tokenAmount, messageFee, arbiterFee, consistencyLevel, nonce) {
    return __awaiter(this, void 0, void 0, function* () {
        const nonceHex = (typeof nonce !== "undefined") ? nonce : utils_1.createNonce().toString('hex');
        return scripts_1.TransferRemote.execute(signerProvider, {
            initialFields: {
                tokenBridge: tokenBridgeId,
                fromAddress: fromAddress,
                tokenPoolId: tokenPoolId,
                remoteTokenId: remoteTokenId,
                tokenChainId: BigInt(tokenChainId),
                toChainId: BigInt(toChainId),
                toAddress: toAddress,
                tokenAmount: tokenAmount,
                arbiterFee: arbiterFee,
                nonce: nonceHex,
                consistencyLevel: BigInt(consistencyLevel)
            },
            attoAlphAmount: messageFee,
            tokens: [{ id: tokenPoolId, amount: tokenAmount }, { id: web3_1.ALPH_TOKEN_ID, amount: web3_1.DUST_AMOUNT * BigInt(2) }]
        });
    });
}
exports.transferRemoteTokenFromAlph = transferRemoteTokenFromAlph;
function getAllowanceEth(tokenBridgeAddress, tokenAddress, signer) {
    return __awaiter(this, void 0, void 0, function* () {
        const token = ethers_contracts_1.TokenImplementation__factory.connect(tokenAddress, signer);
        const signerAddress = yield signer.getAddress();
        const allowance = yield token.allowance(signerAddress, tokenBridgeAddress);
        return allowance;
    });
}
exports.getAllowanceEth = getAllowanceEth;
function approveEth(tokenBridgeAddress, tokenAddress, signer, amount, overrides = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const token = ethers_contracts_1.TokenImplementation__factory.connect(tokenAddress, signer);
        return yield (yield token.approve(tokenBridgeAddress, amount, overrides)).wait();
    });
}
exports.approveEth = approveEth;
function transferFromEth(tokenBridgeAddress, signer, tokenAddress, amount, recipientChain, recipientAddress, relayerFee = 0, overrides = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const recipientChainId = utils_1.coalesceChainId(recipientChain);
        const bridge = ethers_contracts_1.Bridge__factory.connect(tokenBridgeAddress, signer);
        const v = yield bridge.transferTokens(tokenAddress, amount, recipientChainId, recipientAddress, relayerFee, utils_1.createNonce(), overrides);
        const receipt = yield v.wait();
        return receipt;
    });
}
exports.transferFromEth = transferFromEth;
function transferFromEthNative(tokenBridgeAddress, signer, amount, recipientChain, recipientAddress, relayerFee = 0, overrides = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const recipientChainId = utils_1.coalesceChainId(recipientChain);
        const bridge = ethers_contracts_1.Bridge__factory.connect(tokenBridgeAddress, signer);
        const v = yield bridge.wrapAndTransferETH(recipientChainId, recipientAddress, relayerFee, utils_1.createNonce(), Object.assign(Object.assign({}, overrides), { value: amount }));
        const receipt = yield v.wait();
        return receipt;
    });
}
exports.transferFromEthNative = transferFromEthNative;
function transferFromTerra(walletAddress, tokenBridgeAddress, tokenAddress, amount, recipientChain, recipientAddress, relayerFee = "0", payload = null) {
    return __awaiter(this, void 0, void 0, function* () {
        const recipientChainId = utils_1.coalesceChainId(recipientChain);
        const nonce = Math.round(Math.random() * 100000);
        const isNativeAsset = __1.isNativeDenom(tokenAddress);
        const mk_initiate_transfer = (info) => payload
            ? {
                initiate_transfer_with_payload: {
                    asset: {
                        amount,
                        info,
                    },
                    recipient_chain: recipientChainId,
                    recipient: Buffer.from(recipientAddress).toString("base64"),
                    fee: relayerFee,
                    nonce: nonce,
                    payload: payload,
                },
            }
            : {
                initiate_transfer: {
                    asset: {
                        amount,
                        info,
                    },
                    recipient_chain: recipientChainId,
                    recipient: Buffer.from(recipientAddress).toString("base64"),
                    fee: relayerFee,
                    nonce: nonce,
                },
            };
        return isNativeAsset
            ? [
                new terra_js_1.MsgExecuteContract(walletAddress, tokenBridgeAddress, {
                    deposit_tokens: {},
                }, { [tokenAddress]: amount }),
                new terra_js_1.MsgExecuteContract(walletAddress, tokenBridgeAddress, mk_initiate_transfer({
                    native_token: {
                        denom: tokenAddress,
                    },
                }), {}),
            ]
            : [
                new terra_js_1.MsgExecuteContract(walletAddress, tokenAddress, {
                    increase_allowance: {
                        spender: tokenBridgeAddress,
                        amount: amount,
                        expires: {
                            never: {},
                        },
                    },
                }, {}),
                new terra_js_1.MsgExecuteContract(walletAddress, tokenBridgeAddress, mk_initiate_transfer({
                    token: {
                        contract_addr: tokenAddress,
                    },
                }), {}),
            ];
    });
}
exports.transferFromTerra = transferFromTerra;
function transferNativeSol(connection, bridgeAddress, tokenBridgeAddress, payerAddress, amount, targetAddress, targetChain, relayerFee = BigInt(0), payload = null) {
    return __awaiter(this, void 0, void 0, function* () {
        //https://github.com/solana-labs/solana-program-library/blob/master/token/js/client/token.js
        const rentBalance = yield spl_token_1.Token.getMinBalanceRentForExemptAccount(connection);
        const mintPublicKey = new web3_js_1.PublicKey(utils_1.WSOL_ADDRESS);
        const payerPublicKey = new web3_js_1.PublicKey(payerAddress);
        const ancillaryKeypair = web3_js_1.Keypair.generate();
        //This will create a temporary account where the wSOL will be created.
        const createAncillaryAccountIx = web3_js_1.SystemProgram.createAccount({
            fromPubkey: payerPublicKey,
            newAccountPubkey: ancillaryKeypair.publicKey,
            lamports: rentBalance,
            space: spl_token_1.AccountLayout.span,
            programId: spl_token_1.TOKEN_PROGRAM_ID,
        });
        //Send in the amount of SOL which we want converted to wSOL
        const initialBalanceTransferIx = web3_js_1.SystemProgram.transfer({
            fromPubkey: payerPublicKey,
            lamports: Number(amount),
            toPubkey: ancillaryKeypair.publicKey,
        });
        //Initialize the account as a WSOL account, with the original payerAddress as owner
        const initAccountIx = yield spl_token_1.Token.createInitAccountInstruction(spl_token_1.TOKEN_PROGRAM_ID, mintPublicKey, ancillaryKeypair.publicKey, payerPublicKey);
        //Normal approve & transfer instructions, except that the wSOL is sent from the ancillary account.
        const { transfer_native_ix, transfer_native_with_payload_ix, approval_authority_address, } = yield wasm_1.importTokenWasm();
        const nonce = utils_1.createNonce().readUInt32LE(0);
        const transferIx = yield solana_1.getBridgeFeeIx(connection, bridgeAddress, payerAddress);
        const approvalIx = spl_token_1.Token.createApproveInstruction(spl_token_1.TOKEN_PROGRAM_ID, ancillaryKeypair.publicKey, new web3_js_1.PublicKey(approval_authority_address(tokenBridgeAddress)), payerPublicKey, //owner
        [], new spl_token_1.u64(amount.toString(16), 16));
        let messageKey = web3_js_1.Keypair.generate();
        const ix = solana_1.ixFromRust(payload !== null
            ? transfer_native_with_payload_ix(tokenBridgeAddress, bridgeAddress, payerAddress, messageKey.publicKey.toString(), ancillaryKeypair.publicKey.toString(), utils_1.WSOL_ADDRESS, nonce, amount.valueOf(), relayerFee.valueOf(), targetAddress, utils_1.coalesceChainId(targetChain), payload)
            : transfer_native_ix(tokenBridgeAddress, bridgeAddress, payerAddress, messageKey.publicKey.toString(), ancillaryKeypair.publicKey.toString(), utils_1.WSOL_ADDRESS, nonce, amount.valueOf(), relayerFee.valueOf(), targetAddress, utils_1.coalesceChainId(targetChain)));
        //Close the ancillary account for cleanup. Payer address receives any remaining funds
        const closeAccountIx = spl_token_1.Token.createCloseAccountInstruction(spl_token_1.TOKEN_PROGRAM_ID, ancillaryKeypair.publicKey, //account to close
        payerPublicKey, //Remaining funds destination
        payerPublicKey, //authority
        []);
        const { blockhash } = yield connection.getRecentBlockhash();
        const transaction = new web3_js_1.Transaction();
        transaction.recentBlockhash = blockhash;
        transaction.feePayer = new web3_js_1.PublicKey(payerAddress);
        transaction.add(createAncillaryAccountIx);
        transaction.add(initialBalanceTransferIx);
        transaction.add(initAccountIx);
        transaction.add(transferIx, approvalIx, ix);
        transaction.add(closeAccountIx);
        transaction.partialSign(messageKey);
        transaction.partialSign(ancillaryKeypair);
        return transaction;
    });
}
exports.transferNativeSol = transferNativeSol;
function transferFromSolana(connection, bridgeAddress, tokenBridgeAddress, payerAddress, fromAddress, mintAddress, amount, targetAddress, targetChain, originAddress, originChain, fromOwnerAddress, relayerFee = BigInt(0), payload = null) {
    return __awaiter(this, void 0, void 0, function* () {
        const originChainId = originChain
            ? utils_1.coalesceChainId(originChain)
            : undefined;
        const nonce = utils_1.createNonce().readUInt32LE(0);
        const transferIx = yield solana_1.getBridgeFeeIx(connection, bridgeAddress, payerAddress);
        const { transfer_native_ix, transfer_wrapped_ix, transfer_native_with_payload_ix, transfer_wrapped_with_payload_ix, approval_authority_address, } = yield wasm_1.importTokenWasm();
        const approvalIx = spl_token_1.Token.createApproveInstruction(spl_token_1.TOKEN_PROGRAM_ID, new web3_js_1.PublicKey(fromAddress), new web3_js_1.PublicKey(approval_authority_address(tokenBridgeAddress)), new web3_js_1.PublicKey(fromOwnerAddress || payerAddress), [], new spl_token_1.u64(amount.toString(16), 16));
        let messageKey = web3_js_1.Keypair.generate();
        const isSolanaNative = originChainId === undefined || originChainId === utils_1.CHAIN_ID_SOLANA;
        if (!isSolanaNative && !originAddress) {
            throw new Error("originAddress is required when specifying originChain");
        }
        const ix = solana_1.ixFromRust(isSolanaNative
            ? payload !== null
                ? transfer_native_with_payload_ix(tokenBridgeAddress, bridgeAddress, payerAddress, messageKey.publicKey.toString(), fromAddress, mintAddress, nonce, amount.valueOf(), relayerFee.valueOf(), targetAddress, utils_1.coalesceChainId(targetChain), payload)
                : transfer_native_ix(tokenBridgeAddress, bridgeAddress, payerAddress, messageKey.publicKey.toString(), fromAddress, mintAddress, nonce, amount.valueOf(), relayerFee.valueOf(), targetAddress, utils_1.coalesceChainId(targetChain))
            : payload !== null
                ? transfer_wrapped_with_payload_ix(tokenBridgeAddress, bridgeAddress, payerAddress, messageKey.publicKey.toString(), fromAddress, fromOwnerAddress || payerAddress, originChainId, // checked by isSolanaNative
                originAddress, // checked by throw
                nonce, amount.valueOf(), relayerFee.valueOf(), targetAddress, utils_1.coalesceChainId(targetChain), payload)
                : transfer_wrapped_ix(tokenBridgeAddress, bridgeAddress, payerAddress, messageKey.publicKey.toString(), fromAddress, fromOwnerAddress || payerAddress, originChainId, // checked by isSolanaNative
                originAddress, // checked by throw
                nonce, amount.valueOf(), relayerFee.valueOf(), targetAddress, utils_1.coalesceChainId(targetChain)));
        const transaction = new web3_js_1.Transaction().add(transferIx, approvalIx, ix);
        const { blockhash } = yield connection.getRecentBlockhash();
        transaction.recentBlockhash = blockhash;
        transaction.feePayer = new web3_js_1.PublicKey(payerAddress);
        transaction.partialSign(messageKey);
        return transaction;
    });
}
exports.transferFromSolana = transferFromSolana;
/**
 * Transfers an asset from Algorand to a receiver on another chain
 * @param client AlgodV2 client
 * @param tokenBridgeId Application ID of the token bridge
 * @param bridgeId Application ID of the core bridge
 * @param sender Sending account
 * @param assetId Asset index
 * @param qty Quantity to transfer
 * @param receiver Receiving account
 * @param chain Reeiving chain
 * @param fee Transfer fee
 * @param payload payload for payload3 transfers
 * @returns Sequence number of confirmation
 */
function transferFromAlgorand(client, tokenBridgeId, bridgeId, senderAddr, assetId, qty, receiver, chain, fee, payload = null) {
    return __awaiter(this, void 0, void 0, function* () {
        const recipientChainId = utils_1.coalesceChainId(chain);
        const tokenAddr = algosdk_1.getApplicationAddress(tokenBridgeId);
        const applAddr = bridge_1.getEmitterAddressAlgorand(tokenBridgeId);
        const txs = [];
        // "transferAsset"
        const { addr: emitterAddr, txs: emitterOptInTxs } = yield algorand_1.optin(client, senderAddr, bridgeId, BigInt(0), applAddr);
        txs.push(...emitterOptInTxs);
        let creator;
        let creatorAcctInfo;
        let wormhole = false;
        if (assetId !== BigInt(0)) {
            const assetInfo = yield client
                .getAssetByID(bigint_1.safeBigIntToNumber(assetId))
                .do();
            creator = assetInfo["params"]["creator"];
            creatorAcctInfo = yield client.accountInformation(creator).do();
            const authAddr = creatorAcctInfo["auth-addr"];
            if (authAddr === tokenAddr) {
                wormhole = true;
            }
        }
        const params = yield client.getTransactionParams().do();
        const msgFee = yield algorand_1.getMessageFee(client, bridgeId);
        if (msgFee > 0) {
            const payTxn = algosdk_1.makePaymentTxnWithSuggestedParamsFromObject({
                from: senderAddr,
                suggestedParams: params,
                to: algosdk_1.getApplicationAddress(tokenBridgeId),
                amount: msgFee,
            });
            txs.push({ tx: payTxn, signer: null });
        }
        if (!wormhole) {
            const bNat = Buffer.from("native", "binary").toString("hex");
            // "creator"
            const result = yield algorand_1.optin(client, senderAddr, tokenBridgeId, assetId, bNat);
            creator = result.addr;
            txs.push(...result.txs);
        }
        if (assetId !== BigInt(0) &&
            !(yield algorand_1.assetOptinCheck(client, assetId, creator))) {
            // Looks like we need to optin
            const payTxn = algosdk_1.makePaymentTxnWithSuggestedParamsFromObject({
                from: senderAddr,
                to: creator,
                amount: 100000,
                suggestedParams: params,
            });
            txs.push({ tx: payTxn, signer: null });
            // The tokenid app needs to do the optin since it has signature authority
            const bOptin = utils_1.textToUint8Array("optin");
            let txn = algosdk_1.makeApplicationCallTxnFromObject({
                from: senderAddr,
                appIndex: bigint_1.safeBigIntToNumber(tokenBridgeId),
                onComplete: algosdk_1.OnApplicationComplete.NoOpOC,
                appArgs: [bOptin, algosdk_1.bigIntToBytes(assetId, 8)],
                foreignAssets: [bigint_1.safeBigIntToNumber(assetId)],
                accounts: [creator],
                suggestedParams: params,
            });
            txn.fee *= 2;
            txs.push({ tx: txn, signer: null });
        }
        const t = algosdk_1.makeApplicationCallTxnFromObject({
            from: senderAddr,
            appIndex: bigint_1.safeBigIntToNumber(tokenBridgeId),
            onComplete: algosdk_1.OnApplicationComplete.NoOpOC,
            appArgs: [utils_1.textToUint8Array("nop")],
            suggestedParams: params,
        });
        txs.push({ tx: t, signer: null });
        let accounts = [];
        if (assetId === BigInt(0)) {
            const t = algosdk_1.makePaymentTxnWithSuggestedParamsFromObject({
                from: senderAddr,
                to: creator,
                amount: qty,
                suggestedParams: params,
            });
            txs.push({ tx: t, signer: null });
            accounts = [emitterAddr, creator, creator];
        }
        else {
            const t = algosdk_1.makeAssetTransferTxnWithSuggestedParamsFromObject({
                from: senderAddr,
                to: creator,
                suggestedParams: params,
                amount: qty,
                assetIndex: bigint_1.safeBigIntToNumber(assetId),
            });
            txs.push({ tx: t, signer: null });
            accounts = [emitterAddr, creator, creatorAcctInfo["address"]];
        }
        let args = [
            utils_1.textToUint8Array("sendTransfer"),
            algosdk_1.bigIntToBytes(assetId, 8),
            algosdk_1.bigIntToBytes(qty, 8),
            utils_1.hexToUint8Array(receiver),
            algosdk_1.bigIntToBytes(recipientChainId, 8),
            algosdk_1.bigIntToBytes(fee, 8),
        ];
        if (payload !== null) {
            args.push(payload);
        }
        let acTxn = algosdk_1.makeApplicationCallTxnFromObject({
            from: senderAddr,
            appIndex: bigint_1.safeBigIntToNumber(tokenBridgeId),
            onComplete: algosdk_1.OnApplicationComplete.NoOpOC,
            appArgs: args,
            foreignApps: [bigint_1.safeBigIntToNumber(bridgeId)],
            foreignAssets: [bigint_1.safeBigIntToNumber(assetId)],
            accounts: accounts,
            suggestedParams: params,
        });
        acTxn.fee *= 2;
        txs.push({ tx: acTxn, signer: null });
        return txs;
    });
}
exports.transferFromAlgorand = transferFromAlgorand;
